@@ -278,6 +278,13 @@ bool BoundingBoxCameraSensor::CreateCamera()
   auto width = sdfCamera->ImageWidth();
   auto height = sdfCamera->ImageHeight();
 
+  if (width == 0u || height == 0u)
+  {
+    ignerr << "Unable to create a bounding box camera sensor with 0 width or "
+          << "height. " << std::endl;
+    return false;
+  }
+
   // Set Camera Properties
   this->dataPtr->rgbCamera->SetImageFormat(rendering::PF_R8G8B8);
   this->dataPtr->rgbCamera->SetImageWidth(width);
