@@ -35,6 +35,7 @@
 #include <gz/math/Helpers.hh>
 #include <gz/transport/Node.hh>
 
+#include "CameraSensorUtil.hh"
 #include "gz/sensors/CameraSensor.hh"
 #include "gz/sensors/ImageBrownDistortionModel.hh"
 #include "gz/sensors/ImageDistortion.hh"
@@ -69,69 +70,6 @@ class gz::sensors::CameraSensorPrivate
   public: bool SaveImage(const unsigned char *_data, unsigned int _width,
     unsigned int _height, common::Image::PixelFormatType _format);
 
-  /// \brief Computes the OpenGL NDC matrix
-  /// \param[in] _left Left vertical clipping plane
-  /// \param[in] _right Right vertical clipping plane
-  /// \param[in] _bottom Bottom horizontal clipping plane
-  /// \param[in] _top Top horizontal clipping plane
-  /// \param[in] _near Distance to the nearer depth clipping plane
-  ///            This value is negative if the plane is to be behind
-  ///            the camera
-  /// \param[in] _far Distance to the farther depth clipping plane
-  ///            This value is negative if the plane is to be behind
-  ///            the camera
-  /// \return OpenGL NDC (Normalized Device Coordinates) matrix
-  public: static math::Matrix4d BuildNDCMatrix(
-          double _left, double _right,
-          double _bottom, double _top,
-          double _near, double _far);
-
-  /// \brief Computes the OpenGL perspective matrix
-  /// \param[in] _intrinsicsFx Horizontal focal length (in pixels)
-  /// \param[in] _intrinsicsFy Vertical focal length (in pixels)
-  /// \param[in] _intrinsicsCx X coordinate of principal point in pixels
-  /// \param[in] _intrinsicsCy Y coordinate of principal point in pixels
-  /// \param[in] _intrinsicsS Skew coefficient defining the angle between
-  ///            the x and y pixel axes
-  /// \param[in] _clipNear Distance to the nearer depth clipping plane
-  ///            This value is negative if the plane is to be behind
-  ///            the camera
-  /// \param[in] _clipFar Distance to the farther depth clipping plane
-  ///            This value is negative if the plane is to be behind
-  ///            the camera
-  /// \return OpenGL perspective matrix
-  public: static math::Matrix4d BuildPerspectiveMatrix(
-          double _intrinsicsFx, double _intrinsicsFy,
-          double _intrinsicsCx, double _intrinsicsCy,
-          double _intrinsicsS,
-          double _clipNear, double _clipFar);
-
-  /// \brief Computes the OpenGL projection matrix by multiplying
-  ///        the OpenGL Normalized Device Coordinates matrix (NDC) with
-  ///        the OpenGL perspective matrix
-  ///        openglProjectionMatrix = ndcMatrix * perspectiveMatrix
-  /// \param[in] _imageWidth Image width (in pixels)
-  /// \param[in] _imageHeight Image height (in pixels)
-  /// \param[in] _intrinsicsFx Horizontal focal length (in pixels)
-  /// \param[in] _intrinsicsFy Vertical focal length (in pixels)
-  /// \param[in] _intrinsicsCx X coordinate of principal point in pixels
-  /// \param[in] _intrinsicsCy Y coordinate of principal point in pixels
-  /// \param[in] _intrinsicsS Skew coefficient defining the angle between
-  ///             the x and y pixel axes
-  /// \param[in] _clipNear Distance to the nearer depth clipping plane
-  ///            This value is negative if the plane is to be behind
-  ///            the camera
-  /// \param[in] _clipFar Distance to the farther depth clipping plane
-  ///            This value is negative if the plane is to be behind
-  ///            the camera
-  /// \return OpenGL projection matrix
-  public: static math::Matrix4d BuildProjectionMatrix(
-          double _imageWidth, double _imageHeight,
-          double _intrinsicsFx, double _intrinsicsFy,
-          double _intrinsicsCx, double _intrinsicsCy,
-          double _intrinsicsS,
-          double _clipNear, double _clipFar);
-
   /// \brief node to create publisher
   public: transport::Node node;
 
@@ -220,6 +158,13 @@ bool CameraSensor::CreateCamera()
   unsigned int width = cameraSdf->ImageWidth();
   unsigned int height = cameraSdf->ImageHeight();
 
+  if (width == 0u || height == 0u)
+  {
+    ignerr << "Unable to create a camera sensor with 0 width or height."
+          << std::endl;
+    return false;
+  }
+
   this->dataPtr->camera = this->Scene()->CreateCamera(this->Name());
   this->dataPtr->camera->SetImageWidth(width);
   this->dataPtr->camera->SetImageHeight(height);
@@ -316,7 +261,7 @@ bool CameraSensor::CreateCamera()
     double cx = cameraSdf->LensIntrinsicsCx();
     double cy = cameraSdf->LensIntrinsicsCy();
     double s = cameraSdf->LensIntrinsicsSkew();
-    auto projectionMatrix = CameraSensorPrivate::BuildProjectionMatrix(
+    auto projectionMatrix = buildProjectionMatrix(
         this->dataPtr->camera->ImageWidth(),
         this->dataPtr->camera->ImageHeight(),
         fx, fy, cx, cy, s,
@@ -370,7 +315,7 @@ bool CameraSensor::CreateCamera()
     double cy = cameraSdf->LensProjectionCy();
     double s = 0;
 
-    auto projectionMatrix = CameraSensorPrivate::BuildProjectionMatrix(
+    auto projectionMatrix = buildProjectionMatrix(
         this->dataPtr->camera->ImageWidth(),
         this->dataPtr->camera->ImageHeight(),
         fx, fy, cx, cy, s,
@@ -896,72 +841,7 @@ bool CameraSensor::HasInfoConnections() const
 }
 
 //////////////////////////////////////////////////
-math::Matrix4d CameraSensorPrivate::BuildProjectionMatrix(
-    double _imageWidth, double _imageHeight,
-    double _intrinsicsFx, double _intrinsicsFy,
-    double _intrinsicsCx, double _intrinsicsCy,
-    double _intrinsicsS,
-    double _clipNear, double _clipFar)
+const std::string &CameraSensor::OpticalFrameId() const
 {
-  return CameraSensorPrivate::BuildNDCMatrix(
-           0, _imageWidth, 0, _imageHeight, _clipNear, _clipFar) *
-           CameraSensorPrivate::BuildPerspectiveMatrix(
-             _intrinsicsFx, _intrinsicsFy,
-             _intrinsicsCx, _imageHeight - _intrinsicsCy,
-             _intrinsicsS, _clipNear, _clipFar);
-}
-
-//////////////////////////////////////////////////
-math::Matrix4d CameraSensorPrivate::BuildNDCMatrix(
-    double _left, double _right,
-    double _bottom, double _top,
-    double _near, double _far)
-{
-  double inverseWidth = 1.0 / (_right - _left);
-  double inverseHeight = 1.0 / (_top - _bottom);
-  double inverseDistance = 1.0 / (_far - _near);
-
-  return math::Matrix4d(
-           2.0 * inverseWidth,
-           0.0,
-           0.0,
-           -(_right + _left) * inverseWidth,
-           0.0,
-           2.0 * inverseHeight,
-           0.0,
-           -(_top + _bottom) * inverseHeight,
-           0.0,
-           0.0,
-           -2.0 * inverseDistance,
-           -(_far + _near) * inverseDistance,
-           0.0,
-           0.0,
-           0.0,
-           1.0);
-}
-
-//////////////////////////////////////////////////
-math::Matrix4d CameraSensorPrivate::BuildPerspectiveMatrix(
-    double _intrinsicsFx, double _intrinsicsFy,
-    double _intrinsicsCx, double _intrinsicsCy,
-    double _intrinsicsS,
-    double _clipNear, double _clipFar)
-{
-  return math::Matrix4d(
-           _intrinsicsFx,
-           _intrinsicsS,
-           -_intrinsicsCx,
-           0.0,
-           0.0,
-           _intrinsicsFy,
-           -_intrinsicsCy,
-           0.0,
-           0.0,
-           0.0,
-           _clipNear + _clipFar,
-           _clipNear * _clipFar,
-           0.0,
-           0.0,
-           -1.0,
-           0.0);
+  return this->dataPtr->opticalFrameId;
 }
