@@ -14,24 +14,26 @@
  * limitations under the License.
  *
 */
+#if defined(_MSC_VER)
+  #pragma warning(push)
+  #pragma warning(disable: 4005)
+  #pragma warning(disable: 4251)
+#endif
+#include <gz/msgs.hh>
+#if defined(_MSC_VER)
+  #pragma warning(pop)
+#endif
 #include <gtest/gtest.h>
 #include <sdf/sdf.hh>
 
-#include <ignition/math/Angle.hh>
-#include <ignition/math/Helpers.hh>
-#ifdef _WIN32
-#pragma warning(push)
-#pragma warning(disable: 4005)
-#pragma warning(disable: 4251)
-#endif
-#include <ignition/msgs.hh>
-#ifdef _WIN32
-#pragma warning(pop)
-#endif
+#include <gz/math/Angle.hh>
+#include <gz/math/Helpers.hh>
 
-#include <ignition/sensors/Export.hh>
-#include <ignition/sensors/Manager.hh>
-#include <ignition/sensors/Lidar.hh>
+#include <gz/sensors/Export.hh>
+#include <gz/sensors/Manager.hh>
+#include <gz/sensors/Lidar.hh>
+
+using namespace ignition;
 
 sdf::ElementPtr LidarToSDF(const std::string &name, double update_rate,
     const std::string &topic, double horz_samples, double horz_resolution,
@@ -103,7 +105,7 @@ class Lidar_TEST : public ::testing::Test
   // Documentation inherited
   protected: void SetUp() override
   {
-    ignition::common::Console::SetVerbosity(4);
+    gz::common::Console::SetVerbosity(4);
   }
 };
 
@@ -112,11 +114,11 @@ class Lidar_TEST : public ::testing::Test
 TEST(Lidar_TEST, CreateLaser)
 {
   // Create a sensor manager
-  ignition::sensors::Manager mgr;
+  gz::sensors::Manager mgr;
 
   // Create SDF describing a camera sensor
   const std::string name = "TestLidar";
-  const std::string topic = "/ignition/sensors/test/lidar";
+  const std::string topic = "/gz/sensors/test/lidar";
   const double update_rate = 30;
   const double horz_samples = 640;
   const double horz_resolution = 1;
@@ -140,16 +142,18 @@ TEST(Lidar_TEST, CreateLaser)
     visualize);
 
   // Create a CameraSensor
-  ignition::sensors::Lidar *sensor = mgr.CreateSensor<ignition::sensors::Lidar>(
+  gz::sensors::Lidar *sensor = mgr.CreateSensor<gz::sensors::Lidar>(
       lidarSDF);
 
+  EXPECT_FALSE(sensor->CreateLidar());
+
   // Make sure the above dynamic cast worked.
   ASSERT_NE(nullptr, sensor);
 
   double angleRes = (sensor->AngleMax() - sensor->AngleMin()).Radian() /
                     sensor->RayCount();
-  EXPECT_EQ(sensor->AngleMin(), ignition::math::Angle(-1.396263));
-  EXPECT_EQ(sensor->AngleMax(), ignition::math::Angle(1.396263));
+  EXPECT_EQ(sensor->AngleMin(), gz::math::Angle(-1.396263));
+  EXPECT_EQ(sensor->AngleMax(), gz::math::Angle(1.396263));
   EXPECT_NEAR(sensor->RangeMin(), 0.08, 1e-6);
   EXPECT_NEAR(sensor->RangeMax(), 10.0, 1e-6);
   EXPECT_NEAR(sensor->AngleResolution(), angleRes, 1e-3);
@@ -166,6 +170,72 @@ TEST(Lidar_TEST, CreateLaser)
   EXPECT_TRUE(sensor->IsActive());
 }
 
+TEST(Lidar_TEST, CreateLaserFailures)
+{
+  sdf::Sensor sdfSensor;
+  sdfSensor.SetName("camera");
+  sdfSensor.SetType(sdf::SensorType::CAMERA);
+  sdf::Lidar sdfLidarSensor;
+
+  gz::sensors::Lidar sensor;
+  EXPECT_FALSE(sensor.Load(sdfSensor));
+
+  EXPECT_DOUBLE_EQ(0.0, sensor.Range(-1));
+  EXPECT_DOUBLE_EQ(0.0, sensor.Range(0));
+
+  EXPECT_FALSE(sensor.IsHorizontal());
+  EXPECT_DOUBLE_EQ(640, sensor.RangeCountRatio());
+  sensor.SetAngleMax(0.707);
+  sensor.SetAngleMin(-0.707);
+  EXPECT_DOUBLE_EQ(-0.707, sensor.AngleMin().Radian());
+  EXPECT_DOUBLE_EQ(0.707, sensor.AngleMax().Radian());
+
+  sensor.SetVerticalAngleMax(0.707);
+  sensor.SetVerticalAngleMin(-0.707);
+  EXPECT_DOUBLE_EQ(-0.707, sensor.VerticalAngleMin().Radian());
+  EXPECT_DOUBLE_EQ(0.707, sensor.VerticalAngleMax().Radian());
+
+  sensor.ApplyNoise();
+
+  sdfSensor.SetType(sdf::SensorType::LIDAR);
+  sdfSensor.SetLidarSensor(sdfLidarSensor);
+
+  gz::sensors::Lidar sensor2;
+
+  EXPECT_TRUE(sensor2.Load(sdfSensor));
+  EXPECT_FALSE(sensor2.Load(sdfSensor));
+
+  gz::sensors::Lidar sensor3;
+
+  sdfLidarSensor.SetHorizontalScanSamples(0);
+  sdfSensor.SetLidarSensor(sdfLidarSensor);
+  EXPECT_TRUE(sensor3.Load(sdfSensor));
+
+  sdfLidarSensor.SetHorizontalScanSamples(20);
+
+  sdf::Noise noise;
+  noise.SetType(sdf::NoiseType::GAUSSIAN);
+  noise.SetMean(1.2);
+  noise.SetStdDev(2.3);
+  noise.SetBiasMean(4.5);
+  noise.SetBiasStdDev(6.7);
+  noise.SetPrecision(8.9);
+
+  sdfLidarSensor.SetLidarNoise(noise);
+  sdfSensor.SetLidarSensor(sdfLidarSensor);
+  sensors::Lidar sensor4;
+  EXPECT_TRUE(sensor4.Load(sdfSensor));
+
+  noise.SetType(sdf::NoiseType::GAUSSIAN_QUANTIZED);
+  sdfLidarSensor.SetLidarNoise(noise);
+  sdfSensor.SetLidarSensor(sdfLidarSensor);
+  sensors::Lidar sensor5;
+  EXPECT_TRUE(sensor5.Load(sdfSensor));
+
+  sensor.Update(std::chrono::steady_clock::duration(
+    std::chrono::milliseconds(100)));
+}
+
 //////////////////////////////////////////////////
 int main(int argc, char **argv)
 {
