@@ -251,8 +251,15 @@ bool ThermalCameraSensor::CreateCamera()
     return false;
   }
 
-  int width = cameraSdf->ImageWidth();
-  int height = cameraSdf->ImageHeight();
+  unsigned int width = cameraSdf->ImageWidth();
+  unsigned int height = cameraSdf->ImageHeight();
+
+  if (width == 0u || height == 0u)
+  {
+    ignerr << "Unable to create a thermal camera sensor with 0 width or height."
+          << std::endl;
+    return false;
+  }
 
   sdf::PixelFormatType pixelFormat = cameraSdf->PixelFormat();
 
