@@ -48,6 +48,8 @@
 #pragma warning(pop)
 #endif
 
+#include <gz/utils/ExtraTestMacros.hh>
+
 #include "test_config.h"  // NOLINT(build/include)
 #include "TransportTestTools.hh"
 
@@ -503,7 +505,8 @@ void DepthCameraSensorTest::ImagesWithBuiltinSDF(
 }
 
 //////////////////////////////////////////////////
-TEST_P(DepthCameraSensorTest, ImagesWithBuiltinSDF)
+TEST_P(DepthCameraSensorTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(ImagesWithBuiltinSDF))
 {
   ImagesWithBuiltinSDF(GetParam());
 }
