@@ -50,6 +50,8 @@
 #pragma warning(pop)
 #endif
 
+#include <gz/utils/ExtraTestMacros.hh>
+
 #include <sdf/sdf.hh>
 
 #include "test_config.h"  // NOLINT(build/include)
@@ -70,7 +72,7 @@ sdf::ElementPtr GpuLidarToSdf(const std::string &name,
     const double vertResolution, const double vertMinAngle,
     const double vertMaxAngle, const double rangeResolution,
     const double rangeMin, const double rangeMax, const bool alwaysOn,
-    const bool visualize)
+    const bool visualize, const std::string &frameId)
 {
   std::ostringstream stream;
   stream
@@ -79,6 +81,7 @@ sdf::ElementPtr GpuLidarToSdf(const std::string &name,
     << " <model name='m1'>"
     << "  <link name='link1'>"
     << "    <sensor name='" << name << "' type='gpu_lidar'>"
+    << "      <gz_frame_id>" << frameId << "</gz_frame_id>"
     << "      <pose>" << pose << "</pose>"
     << "      <topic>" << topic << "</topic>"
     << "      <updateRate>"<< updateRate <<"</updateRate>"
@@ -186,6 +189,7 @@ void GpuLidarSensorTest::CreateGpuLidar(const std::string &_renderEngine)
   const double rangeMax = 10.0;
   const bool alwaysOn = 1;
   const bool visualize = 1;
+  const std::string frameId = "TestGpuLidar_frame";
 
   // Create sensor description in SDF
   gz::math::Pose3d testPose(gz::math::Vector3d(0, 0, 0.1),
@@ -193,7 +197,7 @@ void GpuLidarSensorTest::CreateGpuLidar(const std::string &_renderEngine)
   sdf::ElementPtr lidarSdf = GpuLidarToSdf(name, testPose, updateRate, topic,
     horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
     vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-    rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+    rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
   // Setup ign-rendering with an empty scene
   auto *engine = gz::rendering::engine(_renderEngine);
@@ -305,6 +309,7 @@ void GpuLidarSensorTest::DetectBox(const std::string &_renderEngine)
   const double rangeMax = 10.0;
   const bool alwaysOn = 1;
   const bool visualize = 1;
+  const std::string frameId = "TestGpuLidar_frame";
 
   // Create sensor SDF
   gz::math::Pose3d testPose(gz::math::Vector3d(0.0, 0.0, 0.1),
@@ -312,7 +317,7 @@ void GpuLidarSensorTest::DetectBox(const std::string &_renderEngine)
   sdf::ElementPtr lidarSdf = GpuLidarToSdf(name, testPose, updateRate, topic,
     horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
     vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-    rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+    rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
   // Create and populate scene
   gz::rendering::RenderEngine *engine =
@@ -337,6 +342,7 @@ void GpuLidarSensorTest::DetectBox(const std::string &_renderEngine)
   visualBox1->AddGeometry(scene->CreateBox());
   visualBox1->SetLocalPosition(box01Pose.Pos());
   visualBox1->SetLocalRotation(box01Pose.Rot());
+  visualBox1->SetUserData("laser_retro", 123);
   root->AddChild(visualBox1);
 
   // Create a sensor manager
@@ -369,6 +375,7 @@ void GpuLidarSensorTest::DetectBox(const std::string &_renderEngine)
   // Sensor 1 should see TestBox1
   EXPECT_DOUBLE_EQ(sensor->Range(0), gz::math::INF_D);
   EXPECT_NEAR(sensor->Range(mid), expectedRangeAtMidPointBox1, LASER_TOL);
+  EXPECT_NEAR(123, sensor->Retro(mid), 1e-1);
   EXPECT_DOUBLE_EQ(sensor->Range(last), gz::math::INF_D);
 
   // Make sure to wait to receive the message
@@ -391,7 +398,7 @@ void GpuLidarSensorTest::DetectBox(const std::string &_renderEngine)
       LASER_TOL);
   EXPECT_DOUBLE_EQ(laserMsgs.back().ranges(last), gz::math::INF_D);
 
-  EXPECT_EQ(laserMsgs.back().frame(), name);
+  EXPECT_EQ(laserMsgs.back().frame(), frameId);
   EXPECT_NEAR(laserMsgs.back().angle_min(), horzMinAngle, 1e-4);
   EXPECT_NEAR(laserMsgs.back().angle_max(), horzMaxAngle, 1e-4);
   EXPECT_NEAR(laserMsgs.back().count(), horzSamples, 1e-4);
@@ -402,6 +409,12 @@ void GpuLidarSensorTest::DetectBox(const std::string &_renderEngine)
   EXPECT_NEAR(laserMsgs.back().range_min(), rangeMin, 1e-4);
   EXPECT_NEAR(laserMsgs.back().range_max(), rangeMax, 1e-4);
 
+  EXPECT_TRUE(laserMsgs.back().has_header());
+  EXPECT_LT(1, laserMsgs.back().header().data().size());
+  EXPECT_EQ("frame_id", laserMsgs.back().header().data(0).key());
+  ASSERT_EQ(1, laserMsgs.back().header().data(0).value().size());
+  EXPECT_EQ(frameId, laserMsgs.back().header().data(0).value(0));
+
   ASSERT_TRUE(!pointMsgs.empty());
   EXPECT_EQ(5, pointMsgs.back().field_size());
   EXPECT_EQ("x", pointMsgs.back().field(0).name());
@@ -417,6 +430,12 @@ void GpuLidarSensorTest::DetectBox(const std::string &_renderEngine)
   EXPECT_FALSE(pointMsgs.back().is_dense());
   EXPECT_EQ(32u * horzSamples * vertSamples, pointMsgs.back().data().size());
 
+  EXPECT_TRUE(pointMsgs.back().has_header());
+  EXPECT_LT(1, pointMsgs.back().header().data().size());
+  EXPECT_EQ("frame_id", pointMsgs.back().header().data(0).key());
+  ASSERT_EQ(1, pointMsgs.back().header().data(0).value().size());
+  EXPECT_EQ(frameId, pointMsgs.back().header().data(0).value(0));
+
   // Clean up rendering ptrs
   visualBox1.reset();
 
@@ -454,6 +473,7 @@ void GpuLidarSensorTest::TestThreeBoxes(const std::string &_renderEngine)
   const double rangeMax = 10.0;
   const bool alwaysOn = 1;
   const bool visualize = 1;
+  const std::string frameId = "TestGpuLidar_frame";
 
   // Create sensor SDF
   gz::math::Pose3d testPose1(gz::math::Vector3d(0, 0, 0.1),
@@ -461,7 +481,7 @@ void GpuLidarSensorTest::TestThreeBoxes(const std::string &_renderEngine)
   sdf::ElementPtr lidarSdf1 = GpuLidarToSdf(name1, testPose1, updateRate,
       topic1, horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
       vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
   // Create a second sensor SDF rotated
   gz::math::Pose3d testPose2(gz::math::Vector3d(0, 0, 0.1),
@@ -469,7 +489,7 @@ void GpuLidarSensorTest::TestThreeBoxes(const std::string &_renderEngine)
   sdf::ElementPtr lidarSdf2 = GpuLidarToSdf(name2, testPose2, updateRate,
       topic2, horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
       vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
   // Create and populate scene
   gz::rendering::RenderEngine *engine =
@@ -610,6 +630,7 @@ void GpuLidarSensorTest::VerticalLidar(const std::string &_renderEngine)
   const double rangeMax = 10.0;
   const bool alwaysOn = 1;
   const bool visualize = 1;
+  const std::string frameId = "TestGpuLidar_frame";
 
   // Create sensor SDF
   gz::math::Pose3d testPose(gz::math::Vector3d(0.25, 0.0, 0.5),
@@ -617,7 +638,7 @@ void GpuLidarSensorTest::VerticalLidar(const std::string &_renderEngine)
   sdf::ElementPtr lidarSdf = GpuLidarToSdf(name, testPose, updateRate, topic,
     horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
     vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-    rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+    rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
   // Create and populate scene
   gz::rendering::RenderEngine *engine =
@@ -737,6 +758,7 @@ void GpuLidarSensorTest::ManualUpdate(const std::string &_renderEngine)
   const double rangeMax = 10.0;
   const bool alwaysOn = 1;
   const bool visualize = 1;
+  const std::string frameId = "TestGpuLidar_frame";
 
   // Create sensor SDF
   gz::math::Pose3d testPose1(gz::math::Vector3d(0, 0, 0.1),
@@ -744,7 +766,7 @@ void GpuLidarSensorTest::ManualUpdate(const std::string &_renderEngine)
   sdf::ElementPtr lidarSdf1 = GpuLidarToSdf(name1, testPose1, updateRate,
       topic1, horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
       vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
   // Create a second sensor SDF at an xy offset of 1
   gz::math::Pose3d testPose2(gz::math::Vector3d(1, 1, 0.1),
@@ -752,7 +774,7 @@ void GpuLidarSensorTest::ManualUpdate(const std::string &_renderEngine)
   sdf::ElementPtr lidarSdf2 = GpuLidarToSdf(name2, testPose2, updateRate,
       topic2, horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
       vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
   // Create and populate scene
   gz::rendering::RenderEngine *engine =
@@ -863,6 +885,7 @@ void GpuLidarSensorTest::Topic(const std::string &_renderEngine)
   const bool alwaysOn = 1;
   const bool visualize = 1;
   auto testPose = gz::math::Pose3d();
+  const std::string frameId = "TestGpuLidar_frame";
 
   // Scene
   auto engine = gz::rendering::engine(_renderEngine);
@@ -884,7 +907,7 @@ void GpuLidarSensorTest::Topic(const std::string &_renderEngine)
     auto lidarSdf = GpuLidarToSdf(name, testPose, updateRate, topic,
       horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
       vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
     auto lidar = mgr.CreateSensor<gz::sensors::GpuLidarSensor>(lidarSdf);
     ASSERT_NE(nullptr, lidar);
@@ -898,7 +921,7 @@ void GpuLidarSensorTest::Topic(const std::string &_renderEngine)
     auto lidarSdf = GpuLidarToSdf(name, testPose, updateRate, topic,
       horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
       vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
     auto lidar = mgr.CreateSensor<gz::sensors::GpuLidarSensor>(lidarSdf);
     ASSERT_NE(nullptr, lidar);
@@ -913,7 +936,7 @@ void GpuLidarSensorTest::Topic(const std::string &_renderEngine)
     auto lidarSdf = GpuLidarToSdf(name, testPose, updateRate, topic,
       horzSamples, horzResolution, horzMinAngle, horzMaxAngle,
       vertSamples, vertResolution, vertMinAngle, vertMaxAngle,
-      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize);
+      rangeResolution, rangeMin, rangeMax, alwaysOn, visualize, frameId);
 
     auto sensor = mgr.CreateSensor<gz::sensors::GpuLidarSensor>(lidarSdf);
     EXPECT_EQ(nullptr, sensor);
@@ -921,57 +944,43 @@ void GpuLidarSensorTest::Topic(const std::string &_renderEngine)
 }
 
 /////////////////////////////////////////////////
-#ifdef __APPLE__
-TEST_P(GpuLidarSensorTest, DISABLED_CreateGpuLidar)
-#else
-TEST_P(GpuLidarSensorTest, CreateGpuLidar)
-#endif
+TEST_P(GpuLidarSensorTest,
+       IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(CreateGpuLidar))
 {
   CreateGpuLidar(GetParam());
 }
 
 /////////////////////////////////////////////////
-#ifdef __APPLE__
-TEST_P(GpuLidarSensorTest, DISABLED_DetectBox)
-#else
-TEST_P(GpuLidarSensorTest, DetectBox)
-#endif
+TEST_P(GpuLidarSensorTest,
+       IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(DetectBox))
 {
   DetectBox(GetParam());
 }
 
 /////////////////////////////////////////////////
-#ifdef __APPLE__
-TEST_P(GpuLidarSensorTest, DISABLED_TestThreeBoxes)
-#else
-TEST_P(GpuLidarSensorTest, TestThreeBoxes)
-#endif
+TEST_P(GpuLidarSensorTest,
+       IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(TestThreeBoxes))
 {
   TestThreeBoxes(GetParam());
 }
 
 /////////////////////////////////////////////////
-#ifdef __APPLE__
-TEST_P(GpuLidarSensorTest, DISABLED_VerticalLidar)
-#else
-TEST_P(GpuLidarSensorTest, VerticalLidar)
-#endif
+TEST_P(GpuLidarSensorTest,
+       IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(VerticalLidar))
 {
   VerticalLidar(GetParam());
 }
 
 /////////////////////////////////////////////////
-#ifdef __APPLE__
-TEST_P(GpuLidarSensorTest, DISABLED_ManualUpdate)
-#else
-TEST_P(GpuLidarSensorTest, ManualUpdate)
-#endif
+TEST_P(GpuLidarSensorTest,
+       IGN_UTILS_TEST_ENABLED_ONLY_ON_LINUX(ManualUpdate))
 {
   ManualUpdate(GetParam());
 }
 
 /////////////////////////////////////////////////
-TEST_P(GpuLidarSensorTest, Topic)
+TEST_P(GpuLidarSensorTest,
+       IGN_UTILS_TEST_DISABLED_ON_WIN32(Topic))
 {
   Topic(GetParam());
 }
